unit MainDms;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlite3conn, sqldb, db, FileUtil, sqlite3dyn;

type

  { TMainDm }

  TMainDm = class(TDataModule)

    SQLite3Connection1: TSQLite3Connection;
    qTemp: TSQLQuery;
    CreationScript: TSQLScript;
    SQLTransaction1: TSQLTransaction;

    qTiles: TSQLQuery;
    qTilesX: TFloatField;
    qTilesY: TFloatField;
    qTilesZ: TFloatField;
    qTilesIMAGE: TBlobField;

    procedure DataModuleDestroy(Sender: TObject);
    procedure qTilesAfterPost(DataSet: TDataSet);
    procedure qTilesPostError(DataSet: TDataSet; E: EDatabaseError;
      var DataAction: TDataAction);
  private
    { private declarations }
    function CheckDB: boolean;
    function TableExists(TableName: String): Boolean;
  public
    { public declarations }
    procedure OpenDB(AFileName: String);
    procedure CreateDB(AFileName: String);
    function DatabaseName: String;
    function Connected: Boolean;
    procedure CloseDB;
    procedure ApplyState(value: TDataSetState);
    procedure ImgClear;
    procedure ImgLoad(FileName: String);
    procedure Apply(ReOpen: Boolean);
  end;

var
  MainDm: TMainDm;

// Quelques explications:
//
// la requête qTiles extrait les données de la tables Tiles et permet leur mise
// à jour. Les données sont chargées dans un cache (de la mémoire allouée à cet
// effet) intégré à l'objet qTiles. Les opérations de mises à jour sont
// enregistrées dans le cache puis appliquées dans la base de données (fichier
// sqlite) avec les appels à ApplyUpdates et validées avec Commit (transaction).
//
// Les requêtes de mises à jour sont aussi dans le composant qTiles, il s'agit
// des propriétés : DeleteSQL, InsertSQL, et UpdateSQL.
//
// Les noms des paramètres doivent suivre des conventions expliquées dans
// la doc de la fcl (chercher le fichier fcl.chm)

implementation

{$R *.lfm}

resourcestring
  NoConnectionName = '(Aucune)';

{ TMainDm }

// Ferme la connection en fin d'exécution
procedure TMainDm.DataModuleDestroy(Sender: TObject);
begin
  if SQLite3Connection1.Connected then
    SQLite3Connection1.Close;
end;

// en cas de réussite à la validation
procedure TMainDm.qTilesAfterPost(DataSet: TDataSet);
begin
//  MainDm.Apply;
end;

// Comportement en cas d'erreur à la validation
procedure TMainDm.qTilesPostError(DataSet: TDataSet; E: EDatabaseError;
  var DataAction: TDataAction);
begin
  qTiles.CancelUpdates;
  SQLTransaction1.Rollback;
  qTiles.Open;
end;

// Vérifie que la table Tiles existe car le fichier ouvert peut concerner
// n'importe quelle base de données a priori
function TMainDm.TableExists(TableName: String): Boolean;
begin
  with qTemp do
  begin
    Close;
    SQL.Text :=
      'SELECT DISTINCT name FROM sqlite_master'#10+
      'WHERE type=''table'''#10+
      'AND NAME = :NAME';
    ParamByName('NAME').AsString := 'tiles';
    Open;
    // S'il existe un enreg alors la table existe
    Result := not IsEmpty;
    Close;
  end;
end;

function TMainDm.CheckDB: boolean;
begin
  Result := TableExists('tiles');
End;

// Ouvre une base existante
// La table Tiles est ouverte
procedure TMainDm.OpenDB(AFileName: String);
begin
  SQLite3Connection1.Close;
  SQLite3Connection1.DatabaseName := AFileName;
  SQLite3Connection1.Open;
  if not CheckDB then
    SQLite3Connection1.Close
  else
    qTiles.Open;
end;

// Création d'une nouvelle base de données, le script de création se trouve
// dans la propriété Script de l'objet CreationScript
procedure TMainDm.CreateDB(AFileName: String);
begin
  SQLite3Connection1.Close;
  SQLite3Connection1.DatabaseName := AFileName;
  SQLite3Connection1.Open;
  CreationScript.ExecuteScript;
  SQLTransaction1.Commit;
end;

// Nom de fichier de la base
function TMainDm.DatabaseName: String;
begin
  if SQLite3Connection1.Connected then
    Result := SQLite3Connection1.DatabaseName
  else
    Result := NoConnectionName;
end;

// Etat de la connection
function TMainDm.Connected: Boolean;
begin
  Result := SQLite3Connection1.Connected;
end;

// Ferme la connection à la base
procedure TMainDm.CloseDB;
begin
  SQLite3Connection1.Connected := False;
end;

// Modifie l'état des champs de données en fonction de l'état (insertion,
// modification...)
procedure TMainDm.ApplyState(value: TDataSetState);
begin
  // Les champs clé sont modifiables en création seulement
  qTilesX.ReadOnly := not (value in [dsInsert]);
  qTilesY.ReadOnly := qTilesX.ReadOnly;
  qTilesZ.ReadOnly := qTilesX.ReadOnly;
  // L'image est modifiable aussi en modification
  qTilesIMAGE.ReadOnly := not (value in [dsInsert, dsEdit]);
end;

// Efface l'image
procedure TMainDm.ImgClear;
begin
  qTilesIMAGE.Clear;
end;

// Charge l'image
procedure TMainDm.ImgLoad(FileName: String);
begin
  qTilesIMAGE.LoadFromFile(FileName);
end;

// Applique les modifications apportées au cache aux données de la base
// et valide (Commit) la transaction pour rendre les modifications permanentes
procedure TMainDm.Apply(ReOpen: Boolean);
begin
  if SQLTransaction1.Active then
  begin
    qTiles.ApplyUpdates;
    SQLTransaction1.Commit;
  end;
  // Après le commit les ensemble de données sont fermés, le code de ces
  // composants est comme cela.
  if ReOpen then
    // on ouvre à nouveau l'ensemble de données si nécessaire
    qTiles.Open;
end;

end.

